from inspect import getmembers
from os import environ
from os.path import sep
import re
from types import MethodType
from nose.plugins import Plugin


def describe(test_case):
    test_case._is_describe = True
    return test_case


class SnakeSpec(Plugin):
    env_option = 'NOSE_SNAKESPEC'

    def __init__(self):
        super(SnakeSpec, self).__init__()
        bdd_pattern = r'(?:^|[\b_\.{sep}-])(?:[Tt]est|[Ii]t|[Ss]hould|[Tt]hen)'.format(sep=sep)
        self._snakespec_pattern = re.compile(bdd_pattern)

    def options(self, parser, env=None):
        env = env or environ
        super(SnakeSpec, self).options(parser, env=env)

        enabled = bool(env.get(self.env_option, False))

        parser.add_option(
            '--snakespec',
            action='store_true',
            default=enabled,
            dest='snakespec',
            help='enable bdd-style testing with snakespec (or set ${}=true'.format(
                self.env_option,
            ),
        )
        parser.add_option(
            '--no-snakespec',
            action='store_false',
            dest='snakespec',
            help='disable bdd-style testing with snakespec',
        )

    def configure(self, options, conf):
        super(SnakeSpec, self).configure(options, conf)
        if options.snakespec:
            self.enabled = True

    def wantMethod(self, method):
        return self._is_test(method) or None

    def loadTestsFromTestCase(self, cls):
        for (name, member) in getmembers(cls, self._is_describe):
            member_name = member.__name__
            descendant = type(member_name, (cls,), member.__dict__.copy())
            delattr(descendant, '_is_describe')
            setattr(cls, member_name, descendant)

            for (key, val) in getmembers(member, self._is_test):
                yield self._get_test_from_case(descendant, val)
            for test_case in self.loadTestsFromTestCase(descendant):
                yield test_case

    def _is_describe(self, obj):
        return getattr(obj, '_is_describe', False)

    def _is_test(self, obj):
        return isinstance(obj, MethodType) and self._snakespec_pattern.search(obj.im_func.func_name)

    def _get_test_from_case(self, test_case, test):
        return test_case(test.im_func.func_name)



from unittest import TestCase
from snakespec.snakespec import describe


class TestNestedExample(TestCase):
    def setUp(self):
        self.foo = 'foo'

    def it_should_run_normally(self):
        self.assertEqual(self.foo, 'foo')

    @describe
    class GivenSomeCircumstance(TestCase):
        def setUp(self):
            super(TestNestedExample.GivenSomeCircumstance, self).setUp()
            self.bar = 'bar'

        def it_should_inherit_setup(self):
            self.assertEqual(self.foo, 'foo')

        def it_should_use_its_own_setup(self):
            self.assertEqual(self.bar, 'bar')

        @describe
        class WhenSomethingHappens(TestCase):
            def setUp(self):
                super(TestNestedExample.GivenSomeCircumstance.WhenSomethingHappens, self).setUp()
                self.foobar = self.foo + self.bar

            def it_should_inherit_setup_from_all_ancestors(self):
                self.assertEqual(self.foobar, 'foobar')



from unittest import TestCase
from hamcrest import *
from snakespec.snakespec import describe


class TestHamcrestExample(TestCase):
    def setUp(self):
        self.array = ['foo']

    def it_should_work_with_hamcrest(self):
        assert_that(self.array, contains('foo'))
        assert_that(self.array, not_(contains('bar')))

    @describe
    class WhenSetupIsNested(TestCase):
        def setUp(self):
            super(TestHamcrestExample.WhenSetupIsNested, self).setUp()
            self.array.append('bar')

        def it_should_inherit_setup(self):
            assert_that('foo', is_in(self.array))

        def it_should_use_its_own_setup(self):
            assert_that(self.array, contains_inanyorder('foo', 'bar'))



from setuptools import setup, find_packages

setup(
    name='snakespec',
    version='0.1.1',
    description='BDD testing for python',
    long_description='A simple, scale-free BDD testing plugin for Python',
    url='https://github.com/iredelmeier/snakespec',
    author='Isobel Redelmeier',
    author_email='iredelmeier@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Operating System :: OS Independent',
    ],
    keywords='test nosetests nose nosetest tdd bdd testing rspec',
    packages=find_packages(exclude=['test*', 'docs']),
    install_requires=['nose'],
    extras_require={
        'dev': ['ipdb', 'ipython', 'pyhamcrest'],
    },
    entry_points={'nose.plugins.0.10': ['snakespec=snakespec.snakespec:SnakeSpec']},
)



import os
from unittest import TestCase
from nose.plugins import PluginTester
from snakespec.snakespec import SnakeSpec


class SnakeSpecTester(PluginTester, TestCase):
    activate = '--snakespec'
    plugins = [SnakeSpec()]


class TestNestedExample(SnakeSpecTester):
    suitepath = os.path.join(os.getcwd(), 'examples', 'test_nested_example.py')

    def test_it_should_run_all_tests(self):
        assert 'Ran 4 tests' in self.output
        assert 'FAILED' not in self.output
        assert 'ERROR' not in self.output


class TestHamcrestExample(SnakeSpecTester):
    suitepath = os.path.join(os.getcwd(), 'examples', 'test_hamcrest_example.py')

    def test_test_it_should_work_with_hamcrest(self):
        assert 'Ran 3 tests' in self.output
        assert 'FAILED' not in self.output
        assert 'ERROR' not in self.output
